package dev.shadowsoffire.apotheosis.adventure.compat;

import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.loot.LootController;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;
import java.util.function.Predicate;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixLootRegistry;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixLootEntry;
import net.minecraft.world.item.ItemStack;
import java.util.function.Consumer;
import net.minecraft.world.entity.player.Player;
import dev.shadowsoffire.gateways.entity.GatewayEntity;
import net.minecraft.server.level.ServerLevel;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import com.mojang.datafixers.kinds.Applicative;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import java.util.function.Function;
import java.util.Objects;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.ServerLevelAccessor;
import dev.shadowsoffire.apotheosis.adventure.boss.BossRegistry;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.Level;
import com.google.common.base.Suppliers;
import dev.shadowsoffire.apotheosis.adventure.boss.ApothBoss;
import java.util.function.Supplier;
import net.minecraft.resources.ResourceLocation;
import java.util.Optional;
import dev.shadowsoffire.gateways.gate.Reward;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.gateways.gate.WaveEntity;

public class GatewaysCompat
{
    public static void register() {
        WaveEntity.CODEC.register(Apotheosis.loc("boss"), (Codec)BossWaveEntity.CODEC);
        Reward.CODEC.register(Apotheosis.loc("affix"), (Codec)RarityAffixItemReward.CODEC);
    }
    
    public static class BossWaveEntity implements WaveEntity
    {
        public static Codec<BossWaveEntity> CODEC;
        private final Optional<ResourceLocation> bossId;
        private final Supplier<ApothBoss> boss;
        
        public BossWaveEntity(final Optional<ResourceLocation> bossId) {
            this.bossId = bossId;
            this.boss = (Supplier<ApothBoss>)Suppliers.memoize(() -> {
                final BossRegistry instance = BossRegistry.INSTANCE;
                Objects.requireNonNull(instance);
                return bossId.map(instance::getValue).orElse(null);
            });
        }
        
        public LivingEntity createEntity(final Level level) {
            final ApothBoss realBoss = (ApothBoss)(this.bossId.isEmpty() ? BossRegistry.INSTANCE.getRandomItem(level.f_46441_) : ((ApothBoss)this.boss.get()));
            if (realBoss == null) {
                return null;
            }
            return (LivingEntity)realBoss.createBoss((ServerLevelAccessor)level, BlockPos.f_121853_, level.f_46441_, 0.0f);
        }
        
        public MutableComponent getDescription() {
            return Component.m_237110_("misc.apotheosis.boss", new Object[] { Component.m_237115_(this.bossId.isEmpty() ? "misc.apotheosis.random" : this.boss.get().getEntity().m_20675_()) });
        }
        
        public boolean shouldFinalizeSpawn() {
            return false;
        }
        
        public Codec<? extends WaveEntity> getCodec() {
            return (Codec<? extends WaveEntity>)BossWaveEntity.CODEC;
        }
        
        public int getCount() {
            return 1;
        }
        
        static {
            BossWaveEntity.CODEC = (Codec<BossWaveEntity>)RecordCodecBuilder.create(inst -> inst.group((App)PlaceboCodecs.nullableField(ResourceLocation.f_135803_, "boss").forGetter(b -> b.bossId)).apply((Applicative)inst, (Function)BossWaveEntity::new));
        }
    }
    
    record RarityAffixItemReward(DynamicHolder<LootRarity> rarity) implements Reward {
        public static Codec<RarityAffixItemReward> CODEC;
        
        public void generateLoot(final ServerLevel level, final GatewayEntity gate, final Player summoner, final Consumer<ItemStack> list) {
            final AffixLootEntry entry = (AffixLootEntry)AffixLootRegistry.INSTANCE.getRandomItem(level.f_46441_, summoner.m_36336_(), new Predicate[] { WeightedDynamicRegistry.IDimensional.matches((Level)level), GameStagesCompat.IStaged.matches(summoner) });
            if (entry == null) {
                AdventureModule.LOGGER.error("Failed to find an affix loot item for a RarityAffixItemReward executing in dimension {} with rarity {}.", (Object)level.m_46472_(), (Object)this.rarity.getId());
                return;
            }
            list.accept(LootController.createLootItem(entry.getStack(), (LootRarity)this.rarity.get(), level.f_46441_));
        }
        
        public void appendHoverText(final Consumer<MutableComponent> list) {
            list.accept(Component.m_237110_("reward.apotheosis.affix", new Object[] { ((LootRarity)this.rarity.get()).toComponent() }));
        }
        
        public Codec<? extends Reward> getCodec() {
            return (Codec<? extends Reward>)RarityAffixItemReward.CODEC;
        }
        
        static {
            RarityAffixItemReward.CODEC = (Codec<RarityAffixItemReward>)RecordCodecBuilder.create(inst -> inst.group((App)RarityRegistry.INSTANCE.holderCodec().fieldOf("rarity").forGetter((Function)RarityAffixItemReward::rarity)).apply((Applicative)inst, (Function)RarityAffixItemReward::new));
        }
    }
}
